<style>
	div#notebook {
	 font-family: sans-serif;
	 font-size: 11.5pt;
	 line-height: 180%;
	 color: #303030;
	 -webkit-font-smoothing: antialiased !important;
	 padding-top: 8px !important;
	}
	.notebook_app {
	 background-color: #ffffff;
	}
	div#notebook_panel {
	 margin-top: 3px;
	}
	body,
	div.body {
	 font-family: sans-serif;
	 font-size: 11.5pt;
	 color: #070707;
	 background-color: #ffffff;
	 -webkit-font-smoothing: subpixel-antialiased !important;
	 background: '' #ffffff;
	 background-size: 118px;
	 background-position-y: 11px;
	 background-position-x: 6px;
	}
	body.notebook_app {
	 padding: 0;
	 background-color: #ffffff;
	 background: #ffffff;
	 padding-right: 0px !important;
	 overflow-y: hidden;
	}
	a {
	 font-family: sans-serif;
	 color: #070707;
	 -webkit-font-smoothing: subpixel-antialiased !important;
	}
	a:hover,
	a:focus {
	 color: #000000;
	 -webkit-font-smoothing: subpixel-antialiased !important;
	}
	.nav > li > a:hover,
	.nav > li > a:focus {
	 text-decoration: none;
	 background-color: transparent !important;
	 border-color: transparent !important;
	}
	.list_header,
	div#notebook_list_header.row.list_header {
	 font-size: 14pt;
	 color: #000000;
	 background-color: transparent;
	 height: 35px;
	}
	i.fa.fa-folder {
	 display: inline-block;
	 font: normal normal normal 14px "FontAwesome";
	 font-family: "FontAwesome" !important;
	 text-rendering: auto;
	 -webkit-font-smoothing: antialiased;
	 font-size: 18px;
	 -moz-osx-font-smoothing: grayscale;
	}
	#running .panel-group .panel .panel-heading {
	 font-size: 14pt;
	 color: #070707;
	 padding: 8px 8px;
	 background: #e4e4e4;
	 background-color: #e4e4e4;
	}
	#running .panel-group .panel .panel-heading a {
	 font-size: 14pt;
	 color: #070707;
	}
	#running .panel-group .panel .panel-heading a:focus,
	#running .panel-group .panel .panel-heading a:hover {
	 font-size: 14pt;
	 color: #070707;
	}
	#running .panel-group .panel .panel-body .list_container .list_item {
	 background: #f4f4f4;
	 background-color: #f4f4f4;
	 padding: 2px;
	 border-bottom: 2px solid #dedede;
	}
	#running .panel-group .panel .panel-body .list_container .list_item:hover {
	 background: #f4f4f4;
	 background-color: #f4f4f4;
	}
	#running .panel-group .panel .panel-body {
	 padding: 2px;
	}
	button#refresh_running_list {
	 border: none !important;
	}
	button#refresh_cluster_list {
	 border: none !important;
	}
	div.running_list_info.toolbar_info {
	 font-size: 15px;
	 padding: 4px 0 4px 0;
	 margin-top: 5px;
	 margin-bottom: 8px;
	 height: 24px;
	 line-height: 24px;
	 text-shadow: none;
	}
	.list_placeholder {
	 font-weight: normal;
	}
	#tree-selector {
	 padding: 0px;
	 border-color: transparent;
	}
	#project_name > ul > li > a > i.fa.fa-home {
	 color: #d31242;
	 font-size: 17pt;
	 display: inline-block;
	 position: static;
	 padding: 0px 0px;
	 font-weight: normal;
	 text-align: center;
	 vertical-align: text-top;
	}
	.fa-folder:before {
	 color: #007dc5;
	}
	.fa-arrow-up:before {
	 font-size: 14px;
	}
	.fa-arrow-down:before {
	 font-size: 14px;
	}
	span#last-modified.btn.btn-xs.btn-default.sort-action:hover .fa,
	span#sort-name.btn.btn-xs.btn-default.sort-action:hover .fa {
	 color: #d31242;
	}
	.folder_icon:before {
	 display: inline-block;
	 font: normal normal normal 14px/1 FontAwesome;
	 font-size: inherit;
	 text-rendering: auto;
	 -webkit-font-smoothing: antialiased;
	 -moz-osx-font-smoothing: grayscale;
	 content: "\f07b";
	 color: #007dc5;
	}
	.notebook_icon:before {
	 display: inline-block;
	 font: normal normal normal 14px/1 FontAwesome;
	 font-size: inherit;
	 text-rendering: auto;
	 -webkit-font-smoothing: antialiased;
	 -moz-osx-font-smoothing: grayscale;
	 content: "\f02d";
	 position: relative;
	 color: #488f0c !important;
	 top: 0px;
	}
	.file_icon:before {
	 display: inline-block;
	 font: normal normal normal 14px/1 FontAwesome;
	 font-size: inherit;
	 text-rendering: auto;
	 -webkit-font-smoothing: antialiased;
	 -moz-osx-font-smoothing: grayscale;
	 content: "\f15b";
	 position: relative;
	 top: 0px;
	 color: #6a737d !important;
	}
	#project_name a {
	 display: inline-flex;
	 padding-left: 7px;
	 margin-left: -2px;
	 text-align: -webkit-auto;
	 vertical-align: baseline;
	 font-size: 18px;
	}
	div#notebook_toolbar div.dynamic-instructions {
	 font-family: sans-serif;
	 font-size: 10.5pt;
	 color: #828892;
	}
	span#login_widget > .button,
	#logout {
	 font-family: "Proxima Nova", sans-serif;
	 color: #070707;
	 background: transparent;
	 background-color: transparent;
	 border: 2px solid #e9e9e9;
	 font-weight: normal;
	 box-shadow: none;
	 text-shadow: none;
	 border-radius: 3px;
	 margin-right: 10px;
	 padding: 2px 7px;
	}
	span#login_widget > .button:hover,
	#logout:hover {
	 color: #d31242;
	 background-color: transparent;
	 background: transparent;
	 border: 2px solid #d31242;
	 background-image: none;
	 box-shadow: none !important;
	 border-radius: 3px;
	}
	span#login_widget > .button:focus,
	#logout:focus,
	span#login_widget > .button.focus,
	#logout.focus,
	span#login_widget > .button:active,
	#logout:active,
	span#login_widget > .button.active,
	#logout.active,
	.open > .dropdown-togglespan#login_widget > .button,
	.open > .dropdown-toggle#logout {
	 color: #000000;
	 background-color: #070707;
	 background: #070707;
	 border-color: #070707;
	 background-image: none;
	 box-shadow: none !important;
	 border-radius: 2px;
	}
	body > #header #header-container {
	 padding-bottom: 0px;
	 padding-top: 4px;
	 box-sizing: border-box;
	 -moz-box-sizing: border-box;
	 -webkit-box-sizing: border-box;
	}
	body > #header {
	 background: #ffffff;
	 background-color: #ffffff;
	 position: relative;
	 z-index: 100;
	}
	.list_container {
	 font-size: 11.5pt;
	 color: #070707;
	 border: none;
	 text-shadow: none !important;
	}
	.list_container > div {
	 border-bottom: 1px solid rgba(61,59,73,.10);
	 font-size: 11.5pt;
	}
	.list_header > div,
	.list_item > div {
	 padding-top: 6px;
	 padding-bottom: 2px;
	 padding-left: 0px;
	}
	.list_header > div .item_link,
	.list_item > div .item_link {
	 margin-left: -1px;
	 vertical-align: middle;
	 line-height: 22px;
	 font-size: 11.5pt;
	}
	.item_icon {
	 color: #007dc5;
	 font-size: 13pt;
	 vertical-align: middle;
	}
	.list_item input:not([type="checkbox"]) {
	 padding-right: 0px;
	 height: 1.75em;
	 width: 25%;
	 margin: 0px 0 0;
	 margin-top: 0px;
	}
	.list_header > div .item_link,
	.list_item > div .item_link {
	 margin-left: -1px;
	 vertical-align: middle;
	 line-height: 1.5em;
	 font-size: 11.5pt;
	 display: inline-table;
	 position: static;
	}
	#button-select-all {
	 height: 34px;
	 min-width: 55px;
	 z-index: 0;
	 border: none !important;
	 padding-top: 0px;
	 padding-bottom: 0px;
	 margin-bottom: 0px;
	 margin-top: 0px;
	 left: -3px;
	 border-radius: 0px !important;
	}
	#button-select-all:focus,
	#button-select-all:active:focus,
	#button-select-all.active:focus,
	#button-select-all.focus,
	#button-select-all:active.focus,
	#button-select-all.active.focus {
	 background-color: #e9e9e9 !important;
	 background: #e9e9e9 !important;
	}
	button#tree-selector-btn {
	 height: 34px;
	 font-size: 10.5pt;
	 border: none;
	 left: 0px;
	 border-radius: 0px !important;
	}
	input#select-all.pull-left.tree-selector {
	 margin-left: 7px;
	 margin-right: 2px;
	 margin-top: 2px;
	 top: 4px;
	}
	input[type="radio"],
	input[type="checkbox"] {
	 margin-top: 1px;
	 line-height: normal;
	}
	.delete-button {
	 border: none !important;
	}
	i.fa.fa-trash {
	 font-size: 13.5pt;
	}
	.list_container a {
	 font-size: 16px;
	 color: #070707;
	 border: none;
	 text-shadow: none !important;
	 font-weight: normal;
	 font-style: normal;
	}
	div.list_container a:hover {
	 color: #000000;
	}
	.list_header > div input,
	.list_item > div input {
	 margin-right: 7px;
	 margin-left: 12px;
	 vertical-align: baseline;
	 line-height: 22px;
	 position: relative;
	 top: -1px;
	}
	div.list_item:hover {
	 background-color: #fbfbfb;
	}
	.breadcrumb > li {
	 font-size: 10.5pt;
	 color: #070707;
	 border: none;
	 text-shadow: none !important;
	}
	.breadcrumb > li + li:before {
	 content: "/\00a0";
	 padding: 0px;
	 color: #070707;
	 font-size: 18px;
	}
	#project_name > .breadcrumb {
	 padding: 0px;
	 margin-bottom: 0px;
	 background-color: transparent;
	 font-weight: normal;
	 margin-top: -2px;
	}
	ul#tabs a {
	 font-family: sans-serif;
	 font-size: 11.5pt !important;
	 font-weight: normal;
	 font-style: normal;
	 text-shadow: none !important;
	}
	.nav-tabs {
	 font-family: sans-serif;
	 font-size: 11.5pt !important;
	 font-weight: normal;
	 font-style: normal;
	 background-color: transparent;
	 border-color: transparent;
	 text-shadow: none !important;
	 border: 2px solid transparent;
	}
	.nav-tabs > li > a:active,
	.nav-tabs > li > a:focus,
	.nav-tabs > li > a:hover,
	.nav-tabs > li.active > a,
	.nav-tabs > li.active > a:focus,
	.nav-tabs > li.active > a:hover,
	.nav-tabs > li.active > a,
	.nav-tabs > li.active > a:hover,
	.nav-tabs > li.active > a:focus {
	 color: #d31242;
	 background-color: transparent;
	 border-color: transparent;
	 border-bottom: 2px solid transparent;
	}
	.nav > li.disabled > a,
	.nav > li.disabled > a:hover {
	 color: #828892;
	}
	.nav-tabs > li > a:before {
	 content: "";
	 position: absolute;
	 width: 100%;
	 height: 2px;
	 bottom: -2px;
	 left: 0;
	 background-color: #d31242;
	 visibility: hidden;
	 -webkit-transform: perspective(0)scaleX(0);
	 transform: perspective(0)scaleX(0);
	 -webkit-transition: ease 220ms;
	 transition: ease 220ms;
	 -webkit-font-smoothing: antialiased !important;
	}
	.nav-tabs > li > a:hover:before {
	 visibility: visible;
	 -webkit-transform: perspective(1)scaleX(1);
	 transform: perspective(1)scaleX(1);
	}
	.nav-tabs > li.active > a:before {
	 content: "";
	 position: absolute;
	 width: 100%;
	 height: 2px;
	 bottom: -2px;
	 left: 0;
	 background-color: #d31242;
	 visibility: visible;
	 -webkit-transform: perspective(1)scaleX(1);
	 transform: perspective(1)scaleX(1);
	 -webkit-font-smoothing: subpixel-antialiased !important;
	}
	#notebook-container {
	 padding: 14px 2px;
	 background-color: #ffffff;
	 min-height: 0px;
	 box-shadow: none;
	 width: 83%;
	 margin-right: auto;
	 margin-left: auto;
	}
	div#ipython-main-app.container {
	 width: 83%;
	 margin-right: auto;
	 margin-left: auto;
	 margin-right: auto;
	 margin-left: auto;
	}
	.container {
	 width: 83%;
	 margin-right: auto;
	 margin-left: auto;
	}
	div#menubar-container {
	 width: 100%;
	 width: 83%;
	}
	div#header-container {
	 width: 83%;
	}
	.notebook_app #header {
	 border: none !important;
	}
	.notebook_app #header,
	.edit_app #header {
	 box-shadow: none !important;
	 background-color: #e8e8e8;
	 background: '' #e8e8e8;
	 background-size: 118px;
	 background-position-y: 11px;
	 background-position-x: 6px;
	}
	#header,
	.edit_app #header {
	 font-family: sans-serif;
	 font-size: 11.5pt;
	 box-shadow: none;
	}
	body > #header .header-bar {
	 width: 100%;
	}
	span.checkpoint_status,
	span.autosave_status {
	 font-size: 9pt;
	 display: none;
	}
	#menubar,
	div#menubar {
	 background-color: #e8e8e8;
	 padding-top: 0px !important;
	 margin-top: 0px !important;
	}
	#menubar .navbar,
	.navbar-default {
	 background-color: #e8e8e8;
	 color: #070707;
	 margin-bottom: 0px;
	 margin-top: 1px;
	 padding-left: 0px;
	}
	div#maintoolbar {
	 position: absolute;
	 width: 90%;
	 margin-left: -10%;
	 padding-right: 8%;
	 float: left;
	 background: #ffffff;
	}
	#maintoolbar {
	 margin-bottom: -3px;
	 margin-top: 0px;
	 border: 0px;
	 min-height: 27px;
	 padding-top: 2px;
	 padding-bottom: 0px;
	}
	#maintoolbar .container {
	 width: 75%;
	 margin-right: auto;
	 margin-left: auto;
	}
	.toolbar .btn-group {
	 margin-top: 0px;
	 margin-left: 9px;
	}
	.toolbar .btn {
	 padding: 3px 7px;
	 font-size: medium;
	}
	.navbar {
	 border: none;
	}
	div.navbar-text,
	.navbar-text,
	.navbar-text.indicator_area,
	p.navbar-text.indicator_area {
	 margin-top: 8px !important;
	 margin-bottom: 0px;
	 color: #d31242;
	}
	.navbar-default {
	 font-family: sans-serif;
	 font-size: 11.5pt;
	 line-height: 1.5em;
	 padding-bottom: 0px;
	}
	.navbar-default .navbar-nav > li > a {
	 font-family: sans-serif;
	 font-size: 11.5pt;
	 color: #070707;
	 display: block;
	 line-height: 1.5em;
	 padding-top: 10px;
	 padding-bottom: 8px;
	}
	.navbar-default .navbar-nav > li > a:hover,
	.navbar-default .navbar-nav > li > a:focus {
	 color: #070707 !important;
	 background-color: #dedede !important;
	 border-color: #dedede !important;
	 line-height: 1.5em;
	 transition: 60ms ease;
	}
	.navbar-default .navbar-nav > .open > a,
	.navbar-default .navbar-nav > .open > a:hover,
	.navbar-default .navbar-nav > .open > a:focus {
	 color: #070707;
	 background-color: #dedede !important;
	 border-color: #dedede !important;
	 line-height: 1.5em;
	 transition: 60ms ease;
	}
	.navbar-nav > li > .dropdown-menu {
	 margin-top: -1px;
	}
	.navbar-nav {
	 margin: 0;
	}
	.navbar-btn.btn-xs {
	 background: transparent !important;
	 color: #070707 !important;
	 border: none !important;
	 font-size: 9pt;
	 margin-top: 4px;
	 margin-bottom: 4px;
	}
	div#notification_notebook.notification_widget.btn.btn-xs.navbar-btn {
	 color: #828892 !important;
	 background: transparent;
	 background-color: transparent;
	 border: none;
	 font-size: 8pt;
	 z-index: 0;
	}
	div.notification_widget.info,
	.notification_widget.info,
	.notification_widget:active:hover,
	.notification_widget.active:hover,
	.open > .dropdown-toggle.notification_widget:hover,
	.notification_widget:active:focus,
	.notification_widget.active:focus,
	.open > .dropdown-toggle.notification_widget:focus,
	.notification_widget:active.focus,
	.notification_widget.active.focus,
	.open > .dropdown-toggle.notification_widget.focus,
	div#notification_notebook.notification_widget.btn.btn-xs.navbar-btn,
	div#notification_notebook.notification_widget.btn.btn-xs.navbar-btn:hover,
	div#notification_notebook.notification_widget.btn.btn-xs.navbar-btn:focus {
	 color: #828892 !important;
	 background-color: #e8e8e8 !important;
	 background: #e8e8e8;
	 border-color: #e8e8e8 !important;
	 padding-bottom: 0px !important;
	 margin-bottom: 0px !important;
	 font-size: 8pt !important;
	 z-index: 0;
	 cursor: pointer !important;
	}
	.notification_widget {
	 z-index: -500;
	 font-size: 8pt !important;
	 background: transparent;
	 background-color: transparent;
	 border: none;
	}
	div.notification_widget.info,
	.notification_widget.info {
	 display: none !important;
	}
	.edit_mode .modal_indicator:before {
	 display: none;
	}
	.command_mode .modal_indicator:before {
	 display: none;
	}
	.item_modified {
	 margin-right: 7px;
	 margin-left: 7px;
	 padding-top: 4px;
	}
	.item_buttons .kernel-name {
	 padding-top: 4px;
	 font-size: 11.5pt;
	 color: #d31242;
	}
	.running_notebook_icon:before {
	 color: #488f0c !important;
	 font: normal normal normal 13pt/1 FontAwesome;
	 font-size: 13pt;
	 text-rendering: auto;
	 -webkit-font-smoothing: antialiased;
	 -moz-osx-font-smoothing: grayscale;
	 content: "\f10c";
	 vertical-align: baseline !important;
	 position: static;
	 display: inherit;
	}
	.item_buttons .running-indicator {
	 padding-top: 3px;
	 color: #488f0c;
	 font-family: sans-serif;
	 font-size: 11.5pt;
	 text-rendering: auto;
	 -webkit-font-smoothing: antialiased;
	}
	#notification_trusted {
	 color: #828892 !important;
	 display: none !important;
	 cursor: pointer !important;
	}
	#modal_indicator {
	 float: right !important;
	 color: #d31242;
	 background: #e8e8e8;
	 background-color: #e8e8e8;
	 margin-top: 8px !important;
	 margin-left: 8px;
	}
	i#kernel_indicator_icon {
	 vertical-align: bottom !important;
	}
	#kernel_indicator {
	 float: right !important;
	 color: #d31242;
	 background: transparent;
	 background-color: transparent;
	 border: none;
	 margin-left: -4px;
	 padding-right: 10px;
	 padding-left: 0px;
	}
	i#kernel_indicator_icon:after {
	 font-family: sans-serif;
	 font-size: 18px;
	 font-style: normal;
	 background-color: transparent;
	 color: #d31242;
	 margin-left: 0px;
	 padding-bottom: 12px;
	 padding-right: 10px;
	 padding-left: 10px;
	 vertical-align: bottom !important;
	}
	#kernel_indicator .kernel_indicator_name {
	 display: none !important;
	 padding-left: 0px;
	 padding-right: 5px;
	 color: #d31242;
	 font-size: 11.5pt !important;
	 vertical-align: inherit;
	 line-height: 1.5em;
	 padding-top: 2px;
	 padding-bottom: 8px;
	}
	.kernel_idle_icon:before {
	 display: inline-block;
	 font: normal normal normal 22px/1 FontAwesome;
	 font-size: 22px;
	 text-rendering: auto;
	 -webkit-font-smoothing: antialiased;
	 cursor: pointer;
	 margin-left: -4px !important;
	 opacity: 0.5;
	 vertical-align: middle;
	 margin-top: -2px;
	 content: "\f1db";
	}
	.kernel_busy_icon:before {
	 display: inline-block;
	 font: normal normal normal 22px/1 FontAwesome;
	 font-size: 22px;
	 -webkit-animation: pulsate 2s infinite ease-out;
	 animation: pulsate 2s infinite ease-out;
	 text-rendering: auto;
	 -webkit-font-smoothing: antialiased;
	 cursor: pointer;
	 margin-left: -4px !important;
	 vertical-align: middle;
	 margin-top: -2px;
	 content: "\f111";
	}
	@-webkit-keyframes pulsate {
	 0% {
	  -webkit-transform: scale(1.0,1.0);
	  opacity: 0.8;
	 }
	 8% {
	  -webkit-transform: scale(1.0,1.0);
	  opacity: 0.8;
	 }
	 50% {
	  -webkit-transform: scale(0.75,0.75);
	  opacity: 0.3;
	 }
	 92% {
	  -webkit-transform: scale(1.0,1.0);
	  opacity: 0.8;
	 }
	 100% {
	  -webkit-transform: scale(1.0,1.0);
	  opacity: 0.8;
	 }
	}
	i#kernel_indicator_icon {
	 vertical-align: sub;
	}
	div.notification_widget.info,
	.notification_widget.info,
	.notification_widget:active:hover,
	.notification_widget.active:hover,
	.open > .dropdown-toggle.notification_widget:hover,
	.notification_widget:active:focus,
	.notification_widget.active:focus,
	.open > .dropdown-toggle.notification_widget:focus,
	.notification_widget:active.focus,
	.notification_widget.active.focus,
	.open > .dropdown-toggle.notification_widget.focus,
	div#notification_notebook.notification_widget.btn.btn-xs.navbar-btn,
	div#notification_notebook.notification_widget.btn.btn-xs.navbar-btn:hover,
	div#notification_notebook.notification_widget.btn.btn-xs.navbar-btn:focus {
	 color: #d31242;
	 background-color: #e8e8e8;
	 border-color: #e8e8e8;
	}
	#notification_area,
	div.notification_area {
	 float: right !important;
	 position: static;
	 cursor: pointer;
	 padding-top: 3px;
	}
	.notification_widget,
	div.notification_widget {
	 margin-right: 0px;
	 margin-left: 0px;
	 padding-right: 0px;
	 vertical-align: text-top !important;
	 margin-top: 6px !important;
	 z-index: 1000;
	}
	#kernel_logo_widget,
	#kernel_logo_widget .current_kernel_logo {
	 display: none;
	}
	i.fa.fa-icon {
	 -webkit-font-smoothing: antialiased;
	 -moz-osx-font-smoothing: grayscale;
	 text-rendering: auto;
	}
	.fa {
	 display: inline-block;
	 font: normal normal normal 14px "FontAwesome";
	 font-family: "FontAwesome" !important;
	 text-rendering: auto;
	 -webkit-font-smoothing: antialiased;
	 -moz-osx-font-smoothing: grayscale;
	}
	i#notebook-trusted-indicator.fa.fa-question.notebook-trusted {
	 color: #828892 !important;
	 display: none !important;
	 font-size: 11.5pt !important;
	 vertical-align: inherit;
	 line-height: 1.5em;
	 padding-top: 2px;
	 padding-bottom: 8px;
	}
	.dropdown-menu {
	 font-family: sans-serif;
	 font-size: 11.5pt;
	 box-shadow: none;
	 padding: 0px;
	 text-align: left;
	 border: 0px solid transparent;
	 border-radius: 3px;
	 background-color: #dedede;
	 background: #dedede;
	 line-height: 1.2;
	}
	.dropdown-menu:hover {
	 font-family: sans-serif;
	 font-size: 11.5pt;
	 box-shadow: none;
	 padding: 0px;
	 text-align: left;
	 border: 0px solid #dedede;
	 border-radius: 3px;
	 background-color: #dedede;
	 background: #dedede;
	 box-shadow: none;
	 line-height: 1.2;
	}
	.dropdown-menu > li > a {
	 font-family: sans-serif;
	 font-size: 10.5pt;
	 display: block;
	 padding: 10px 20px 9px 10px;
	 color: #070707;
	 background-color: #dedede;
	 background: #dedede;
	}
	.dropdown-menu > li > a:hover,
	.dropdown-menu > li > a:focus {
	 color: #000000;
	 background-color: #d4d4d4;
	 background: #d4d4d4;
	 border-color: #d4d4d4;
	 transition: 155ms ease;
	}
	.dropdown-menu .divider {
	 height: 1px;
	 margin: 0px 0px;
	 overflow: hidden;
	 background-color: rgba(61,59,73,.15);
	}
	.dropdown-submenu > .dropdown-menu {
	 top: 0px;
	 left: 100%;
	 margin-top: 0px !important;
	 margin-left: 0px;
	 padding-top: 0px;
	 transition: 155ms ease;
	}
	.dropdown-menu > .disabled > a,
	.dropdown-menu > .disabled > a:hover,
	.dropdown-menu > .disabled > a:focus {
	 font-family: sans-serif;
	 font-size: 10.5pt;
	 font-weight: normal;
	 color: #828892;
	 padding: none;
	 display: block;
	 clear: both;
	 white-space: nowrap;
	}
	.dropdown-submenu > a:after {
	 color: #070707;
	 margin-right: -16px;
	 margin-top: 3px !important;
	}
	.dropdown-submenu:hover > a:after,
	.dropdown-submenu:active > a:after,
	.dropdown-submenu:focus > a:after,
	.dropdown-submenu:visited > a:after {
	 color: #d31242;
	 margin-right: -16px;
	}
	div.kse-dropdown > .dropdown-menu,
	.kse-dropdown > .dropdown-menu {
	 min-width: 0;
	 top: 94%;
	}
	.navbar-default .navbar-nav > li:hover > a:before,
	.navbar-default .navbar-nav > li.active > a:before {
	 visibility: hidden;
	}
	ul#new-menu.dropdown-menu,
	ul#selector-menu.dropdown-menu,
	ul#selector-menu.dropdown-menu > li > a,
	ul#new-menu.dropdown-menu > li > a {
	 background: #dedede;
	 background-color: #dedede;
	}
	ul#selector-menu.dropdown-menu > li > a:hover,
	ul#new-menu.dropdown-menu > li > a:hover,
	ul#selector-menu.dropdown-menu > li > a:focus,
	ul#new-menu.dropdown-menu > li > a:focus {
	 background-color: #d4d4d4 !important;
	}
	.btn,
	.btn-default {
	 font-family: sans-serif;
	 color: #070707;
	 background: #e9e9e9;
	 background-color: #e9e9e9;
	 border: 2px solid #e9e9e9;
	 font-weight: normal;
	 box-shadow: none;
	 text-shadow: none;
	 border-radius: 3px;
	 font-size: initial;
	}
	.btn:hover,
	.btn:active:hover,
	.btn.active:hover,
	.btn-default:hover,
	.open > .dropdown-toggle.btn-default:hover,
	.open > .dropdown-toggle.btn:hover {
	 color: #d31242;
	 border: 2px solid #e4e4e4;
	 background-color: #e4e4e4;
	 background: #e4e4e4;
	 background-image: none;
	 box-shadow: none !important;
	 border-radius: 3px;
	}
	.btn:active,
	.btn.active,
	.btn:active:focus,
	.btn.active:focus,
	.btn:active.focus,
	.btn.active.focus,
	.btn-default:focus,
	.btn-default.focus,
	.btn-default:active,
	.btn-default.active,
	.btn-default:active:hover,
	.btn-default.active:hover,
	.btn-default:active:focus,
	.btn-default.active:focus,
	.btn-default:active.focus,
	.btn-default.active.focus,
	.open > .dropdown-toggle.btn:focus,
	.open > .dropdown-toggle.btn.focus,
	.open > .dropdown-toggle.btn-default {
	 color: #d31242;
	 border: 2px solid #e4e4e4;
	 background-color: #e4e4e4;
	 background: #e4e4e4;
	 background-image: none;
	 box-shadow: none !important;
	 border-radius: 3px;
	}
	.btn-default:active:hover,
	.btn-default.active:hover,
	.open > .dropdown-toggle.btn-default:hover,
	.btn-default:active:focus,
	.btn-default.active:focus,
	.open > .dropdown-toggle.btn-default:focus,
	.btn-default:active.focus,
	.btn-default.active.focus,
	.open > .dropdown-toggle.btn-default.focus {
	 color: #d31242 !important;
	 background-color: #e9e9e9;
	 border-color: #f2ced7 !important;
	 transition: 2000ms ease;
	}
	.btn:focus,
	.btn.focus,
	.btn:active:focus,
	.btn.active:focus,
	.btn:active,
	.btn.active,
	.btn:active.focus,
	.btn.active.focus {
	 color: #d31242 !important;
	 outline: none !important;
	 outline-width: 0px !important;
	 background: #f2ced7 !important;
	 background-color: #f2ced7 !important;
	 border-color: #f2ced7 !important;
	 transition: 155ms ease !important;
	}
	.item_buttons > .btn,
	.item_buttons > .btn-group,
	.item_buttons > .input-group {
	 font-size: 11.5pt;
	 background: transparent;
	 background-color: transparent;
	 border: 0px solid #e4e4e4;
	 border-bottom: 2px solid transparent;
	 margin-left: 5px;
	 padding-top: 4px !important;
	}
	.item_buttons > .btn:hover,
	.item_buttons > .btn-group:hover,
	.item_buttons > .input-group:hover,
	.item_buttons > .btn.active,
	.item_buttons > .btn-group.active,
	.item_buttons > .input-group.active,
	.item_buttons > .btn.focus {
	 margin-left: 5px;
	 background: #dfdfdf;
	 padding-top: 4px !important;
	 background-color: transparent;
	 border: 0px solid transparent;
	 border-bottom: 2px solid #d31242;
	 border-radius: 0px;
	 transition: none;
	}
	.item_buttons {
	 line-height: 1.5em !important;
	}
	.item_buttons .btn {
	 min-width: 11ex;
	}
	.btn-group > .btn:first-child {
	 margin-left: 3px;
	}
	.btn-group > .btn-mini,
	.btn-sm,
	.btn-group-sm > .btn,
	.btn-xs,
	.btn-group-xs > .btn,
	.alternate_upload .btn-upload,
	.btn-group,
	.btn-group-vertical {
	 font-size: inherit;
	 font-weight: normal;
	 height: inherit;
	 line-height: inherit;
	}
	.btn-xs,
	.btn-group-xs > .btn {
	 font-size: initial !important;
	 background-image: none;
	 font-weight: normal;
	 text-shadow: none;
	 display: inline-table;
	 padding: 2px 5px;
	 line-height: 1.45;
	}
	.btn-group > .btn:first-child {
	 margin-left: 3px;
	}
	div#new-buttons > button,
	#new-buttons > button,
	div#refresh_notebook_list,
	#refresh_notebook_list {
	 background: transparent;
	 background-color: transparent;
	 border: none;
	}
	div#new-buttons > button:hover,
	#new-buttons > button:hover,
	div#refresh_notebook_list,
	#refresh_notebook_list,
	div.alternate_upload .btn-upload,
	.alternate_upload .btn-upload,
	div.dynamic-buttons > button,
	.dynamic-buttons > button,
	.dynamic-buttons > button:focus,
	.dynamic-buttons > button:active:focus,
	.dynamic-buttons > button.active:focus,
	.dynamic-buttons > button.focus,
	.dynamic-buttons > button:active.focus,
	.dynamic-buttons > button.active.focus,
	#new-buttons > button:focus,
	#new-buttons > button:active:focus,
	#new-buttons > button.active:focus,
	#new-buttons > button.focus,
	#new-buttons > button:active.focus,
	#new-buttons > button.active.focus,
	.alternate_upload .btn-upload:focus,
	.alternate_upload .btn-upload:active:focus,
	.alternate_upload .btn-upload.active:focus,
	.alternate_upload .btn-upload.focus,
	.alternate_upload .btn-upload:active.focus,
	.alternate_upload .btn-upload.active.focus {
	 background: transparent !important;
	 background-color: transparent !important;
	 border: none !important;
	}
	.alternate_upload input.fileinput {
	 text-align: center;
	 vertical-align: bottom;
	 margin-left: -.5ex;
	 display: inline-table;
	 border: solid 0px #e9e9e9;
	 margin-bottom: -1ex;
	}
	.alternate_upload .btn-upload {
	 display: inline-table;
	 background: transparent;
	 border: none;
	}
	.btn-group .btn + .btn,
	.btn-group .btn + .btn-group,
	.btn-group .btn-group + .btn,
	.btn-group .btn-group + .btn-group {
	 margin-left: -2px;
	}
	.btn-group > .btn:first-child:not(:last-child):not(.dropdown-toggle) {
	 border-bottom-right-radius: 0;
	 border-top-right-radius: 0;
	 z-index: 2;
	}
	.dropdown-header {
	 font-family: sans-serif !important;
	 font-size: 11.5pt !important;
	 color: #d31242 !important;
	 border-bottom: none !important;
	 padding: 0px !important;
	 margin: 6px 6px 0px !important;
	}
	span#last-modified.btn.btn-xs.btn-default.sort-action,
	span#sort-name.btn.btn-xs.btn-default.sort-action,
	span#file-size.btn.btn-xs.btn-default.sort-action {
	 font-family: sans-serif;
	 font-size: 16px;
	 background-color: transparent;
	 background: transparent;
	 border: none;
	 color: #070707;
	 padding-bottom: 0px;
	 margin-bottom: 0px;
	 vertical-align: sub;
	}
	span#last-modified.btn.btn-xs.btn-default.sort-action {
	 margin-left: 19px;
	}
	button.close {
	 border: 0px none;
	 font-family: sans-serif;
	 font-size: 20pt;
	 font-weight: normal;
	}
	.dynamic-buttons {
	 padding-top: 0px;
	 display: inline-block;
	}
	.close {
	 color: #b9002d;
	 opacity: .5;
	 text-shadow: none;
	 font-weight: normal;
	}
	.close:hover {
	 color: #b9002d;
	 opacity: 1;
	 font-weight: normal;
	}
	div.nbext-enable-btns .btn[disabled],
	div.nbext-enable-btns .btn[disabled]:hover,
	.btn-default.disabled,
	.btn-default[disabled],
	.btn-default.disabled:hover,
	.btn-default[disabled]:hover,
	fieldset[disabled] .btn-default:hover,
	.btn-default.disabled:focus,
	.btn-default[disabled]:focus,
	fieldset[disabled] .btn-default:focus,
	.btn-default.disabled.focus,
	.btn-default[disabled].focus,
	fieldset[disabled] .btn-default.focus {
	 color: #4a4a4a;
	 background: #e6e6e6;
	 background-color: #e6e6e6;
	 border-color: #e6e6e6;
	 transition: 155ms ease;
	}
	.input-group-addon {
	 padding: 2px 5px;
	 font-size: 11.5pt;
	 font-weight: normal;
	 height: auto;
	 color: #070707;
	 text-align: center;
	 background-color: transparent;
	 border: 2px solid transparent !important;
	 text-transform: capitalize;
	}
	a.btn.btn-default.input-group-addon:hover {
	 background: transparent !important;
	 background-color: transparent !important;
	}
	.btn-group > .btn + .dropdown-toggle {
	 padding-left: 8px;
	 padding-right: 8px;
	 height: 100%;
	}
	.btn-group > .btn + .dropdown-toggle:hover {
	 background: #f2ced7 !important;
	}
	.input-group-btn {
	 position: relative;
	 font-size: inherit;
	 white-space: nowrap;
	 border: none;
	}
	.input-group-btn:hover {
	 border: none;
	}
	.input-group-btn:first-child > .btn,
	.input-group-btn:first-child > .btn-group {
	 background: #e4e4e4;
	 background-color: #e4e4e4;
	 border: none;
	 margin-left: 2px;
	 margin-right: -1px;
	 font-size: inherit;
	}
	.input-group-btn:first-child > .btn:hover,
	.input-group-btn:first-child > .btn-group:hover {
	 background: #e4e4e4;
	 background-color: #e4e4e4;
	 border: none;
	 font-size: inherit;
	 transition: 155ms ease;
	}
	div.modal .btn-group > .btn:first-child {
	 background: #e4e4e4;
	 background-color: #e4e4e4;
	 border: 1px solid #e1e1e1;
	 margin-top: 0px !important;
	 margin-left: 0px;
	 margin-bottom: 2px;
	}
	div.modal .btn-group > .btn:first-child:hover {
	 background: #dfdfdf;
	 background-color: #dfdfdf;
	 border: 1px solid #dfdfdf;
	 transition: 155ms ease;
	}
	div.modal > button,
	div.modal-footer > button {
	 background: #e4e4e4;
	 background-color: #e4e4e4;
	 border-color: #e4e4e4;
	}
	div.modal > button:hover,
	div.modal-footer > button:hover {
	 background: #dfdfdf;
	 background-color: #dfdfdf;
	 border-color: #dfdfdf;
	 transition: 155ms ease;
	}
	.modal-content {
	 font-family: sans-serif;
	 font-size: 10.5pt;
	 position: relative;
	 background: #e4e4e4;
	 background-color: #e4e4e4;
	 border: none;
	 border-radius: 1px;
	 background-clip: padding-box;
	 outline: none;
	}
	.modal-header {
	 font-family: sans-serif;
	 font-size: 11.5pt;
	 color: #070707;
	 background: #e4e4e4;
	 background-color: #e4e4e4;
	 border-color: #dedede;
	 padding: 12px;
	 min-height: 16.4286px;
	}
	.modal-content h4 {
	 font-family: sans-serif;
	 font-size: 16pt;
	 color: #070707;
	 padding: 5px;
	}
	.modal-body {
	 background-color: #ffffff;
	 position: relative;
	 padding: 15px;
	}
	.modal-footer {
	 padding: 8px;
	 text-align: right;
	 background-color: #ffffff;
	 border-top: none;
	}
	.alert-info {
	 background-color: #fdfdfd;
	 border-color: #dedede;
	 color: #070707;
	}
	.modal-header .close {
	 margin-top: -5px;
	 font-size: 25pt;
	}
	.modal-backdrop,
	.modal-backdrop.in {
	 opacity: 0.85;
	 background-color: notebook-bg;
	}
	div.panel,
	div.panel-default,
	.panel,
	.panel-default {
	 font-family: sans-serif;
	 font-size: 11.5pt;
	 background-color: #f4f4f4;
	 color: #070707;
	 margin-bottom: 14px;
	 border: 0;
	 box-shadow: none;
	}
	div.panel > .panel-heading,
	div.panel-default > .panel-heading {
	 font-size: 14pt;
	 color: #070707;
	 background: #e4e4e4;
	 background-color: #e4e4e4;
	 border: 0;
	}
	.modal .modal-dialog {
	 min-width: 950px;
	 margin: 50px auto;
	}
	div.container-fluid {
	 margin-right: auto;
	 margin-left: auto;
	 padding-left: 0px;
	 padding-right: 5px;
	}
	div.form-control,
	.form-control {
	 font-family: sans-serif;
	 font-size: inherit;
	 color: #070707;
	 background-color: #ffffff;
	 border: 1px solid #d0d0d0 !important;
	 margin-left: 2px;
	 box-shadow: none;
	 transition: border-color 0.15s ease-in-out 0s, box-shadow 0.15s ease-in-out 0s;
	}
	.form-control-static {
	 min-height: inherit;
	 height: inherit;
	}
	.form-group.list-group-item {
	 color: #070707;
	 background-color: #f4f4f4;
	 border-color: #dedede;
	 margin-bottom: 0px;
	}
	input,
	button,
	select,
	textarea {
	 background-color: #ffffff;
	 font-weight: normal;
	 border: 1px solid #dedede;
	}
	select.form-control.select-xs {
	 height: 33px;
	 font-size: 11.5pt;
	}
	.toolbar select,
	.toolbar label {
	 width: auto;
	 vertical-align: middle;
	 margin-right: 0px;
	 margin-bottom: 0px;
	 display: inline;
	 font-size: 92%;
	 margin-left: 10px;
	 padding: 0px;
	 background: #e9e9e9 !important;
	 background-color: #e9e9e9 !important;
	 border: 2px solid #e9e9e9 !important;
	}
	.form-control:focus {
	 border-color: #d31242;
	 outline: 2px solid rgba(191,11,55,.30);
	 -webkit-box-shadow: none;
	 box-shadow: none;
	}
	::-webkit-input-placeholder {
	 color: #828892;
	}
	::-moz-placeholder {
	 color: #828892;
	}
	:-ms-input-placeholder {
	 color: #828892;
	}
	:-moz-placeholder {
	 color: #828892;
	}
	[dir="ltr"] #find-and-replace .input-group-btn + .form-control {
	 border: 2px solid #dedede !important;
	}
	[dir="ltr"] #find-and-replace .input-group-btn + .form-control:focus {
	 border-color: #d31242;
	 outline: 2px solid rgba(191,11,55,.30);
	 -webkit-box-shadow: none;
	 box-shadow: none;
	}
	div.output.output_scroll {
	 box-shadow: none;
	}
	::-webkit-scrollbar {
	 width: 11px;
	 max-height: 9px;
	 background-color: #ebebeb;
	 border-radius: 3px;
	 border: none;
	}
	::-webkit-scrollbar-track {
	 background: #ebebeb;
	 border: none;
	 width: 11px;
	 max-height: 9px;
	}
	::-webkit-scrollbar-thumb {
	 border-radius: 2px;
	 border: none;
	 background: #909090;
	 background-clip: content-box;
	 width: 11px;
	}
	HTML,
	body,
	div,
	dl,
	dt,
	dd,
	ul,
	ol,
	li,
	h1,
	h2,
	h3,
	h4,
	h5,
	h6,
	pre,
	code,
	form,
	fieldset,
	legend,
	input,
	button,
	textarea,
	p,
	blockquote,
	th,
	td,
	span,
	a {
	 text-rendering: auto;
	 -webkit-font-smoothing: antialiased;
	}
	.jupyter-dashboard-menu-item.selected::before {
	 font-family: FontAwesome;
	 content: '\f00c';
	 position: absolute;
	 font-size: xx-small;
	 color: #d31242;
	 top: 13px !important;
	 left: 0px !important;
	}
	div.input_area {
	 background-color: #f1f1f1;
	 background: #f1f1f1;
	 padding-right: 1.2em;
	 border: 0px;
	 border-radius: 2px;
	}
	div.cell {
	 padding: 0px;
	 background: #f1f1f1;
	 background-color: #f1f1f1;
	 border: medium solid #ffffff;
	 border-radius: 3px;
	 top: 0;
	}
	div.cell.selected {
	 background: #f1f1f1;
	 background-color: #f1f1f1;
	 border: medium solid #ffffff;
	 padding: 0px;
	 border-radius: 5px;
	}
	.edit_mode div.cell.selected {
	 padding: 0px;
	 background: #f1f1f1;
	 background-color: #f1f1f1;
	 border: medium solid #ffffff;
	 border-radius: 5px;
	}
	div.cell.edit_mode {
	 padding: 0px;
	 background: #f1f1f1;
	 background-color: #f1f1f1;
	}
	div.CodeMirror-sizer {
	 margin-left: 0px;
	 margin-bottom: -21px;
	 border-right-width: 16px;
	 min-height: 37px;
	 padding-right: 0px;
	 padding-bottom: 0px;
	 margin-top: 0px;
	}
	div.cell.selected:before {
	 background: #f1f1f1;
	 border: none;
	 border-radius: 2px;
	 position: absolute;
	 display: block;
	 top: 0px;
	 left: 0px;
	 width: 0px;
	 height: 100%;
	}
	.edit_mode div.cell.selected:before {
	 background: #ffffff;
	 border: none;
	 border-radius: 2px;
	 position: absolute;
	 display: block;
	 top: 0px;
	 left: 0px;
	 width: 0px;
	 height: 100%;
	}
	div.cell.code_cell .input {
	 border-left: 4px solid #f1f1f1 !important;
	 border-radius: 2px;
	}
	div.cell.code_cell.selected .input {
	 border-left: 4px solid #c60030 !important;
	 border-radius: 2px;
	}
	.edit_mode div.cell.code_cell.selected .input {
	 border-left: 4px solid #ffbbc3 !important;
	 border-radius: 2px;
	}
	div.cell.selected:before,
	.edit_mode div.cell.selected:before,
	div.cell.selected:before,
	div.cell.selected.jupyter-soft-selected:before {
	 background: #f1f1f1 !important;
	 border: none;
	 border-radius: 2px;
	 position: absolute;
	 display: block;
	 top: 0px;
	 left: 0px;
	 width: 0px;
	 height: 100%;
	}
	div.cell.text_cell.selected::before,
	.edit_mode div.cell.text_cell.selected:before,
	div.cell.text_cell.selected:before,
	div.cell.text_cell.selected.jupyter-soft-selected:before {
	 background: #ffffff !important;
	 background-color: #ffffff !important;
	 border-color: #e0e0e0 !important;
	}
	.edit_mode div.cell.selected:before {
	 height: 100%;
	 border-left: 4px solid #ffbbc3 !important;
	 border-radius: 2px;
	}
	div.cell.jupyter-soft-selected,
	div.cell.selected.jupyter-soft-selected {
	 border-left-color: #ffbbc3 !important;
	 border-left-width: 0px !important;
	 padding-left: 7px !important;
	 border-right-color: #ffbbc3 !important;
	 border-right-width: 0px !important;
	 background: #ffbbc3 !important;
	 border-radius: 6px !important;
	}
	div.cell.selected.jupyter-soft-selected .input {
	 border-left: 4px solid #f1f1f1 !important;
	}
	div.cell.selected.jupyter-soft-selected:before,
	.edit_mode div.cell.selected:before .input {
	 background: #ffbbc3 !important;
	 border: 2px solid #ffbbc3 !important;
	 border-radius: 6px !important;
	 position: absolute !important;
	 display: block !important;
	 top: 0px !important;
	 left: 0px !important;
	 width: 6px !important;
	 height: 100% !important;
	}
	div.cell.selected.jupyter-soft-selected {
	 border-left-color: #c60030;
	 border-color: #ffffff;
	 padding-left: 7px;
	 border-radius: 6px;
	}
	div.cell.code_cell.selected .input {
	 border-left: none;
	 border-radius: 2px;
	}
	div.cell.selected.jupyter-soft-selected .prompt,
	div.cell.text_cell.selected.jupyter-soft-selected .prompt {
	 top: 0;
	 border-left: #f1f1f1 !important;
	 border-radius: 2px;
	}
	div.cell.text_cell.selected.jupyter-soft-selected .input_prompt {
	 border-left: none !important;
	}
	div.cell.text_cell.selected.jupyter-soft-selected:before {
	 background: #e8e8e8 !important;
	 border: 2px solid #e8e8e8 !important;
	 border-radius: 6px !important;
	 position: absolute !important;
	 display: block !important;
	 top: 0px !important;
	 left: 0px !important;
	 width: 6px !important;
	 height: 100% !important;
	}
	div.cell.text_cell.jupyter-soft-selected,
	div.cell.text_cell.selected.jupyter-soft-selected {
	 border-left-color: #e8e8e8 !important;
	 border-left-width: 0px !important;
	 padding-left: 7px !important;
	 border-right-color: #e8e8e8 !important;
	 border-right-width: 0px !important;
	 background: #e8e8e8 !important;
	 border-radius: 6px !important;
	}
	div.cell.jupyter-soft-selected .input,
	div.cell.selected.jupyter-soft-selected .input {
	 border-left-color: #ffbbc3 !important;
	}
	div.prompt,
	.prompt {
	 font-family: monospace, monospace;
	 font-size: 9pt !important;
	 font-weight: normal;
	 color: transparent;
	 line-height: 170%;
	 padding: 0px;
	 padding-top: 4px;
	 padding-left: 0px;
	 padding-right: 1px;
	 text-align: right !important;
	 min-width: 8ex !important;
	 width: 8ex !important;
	}
	div.prompt.input_prompt {
	 font-size: 9pt !important;
	 background-color: #f1f1f1;
	 border-top: 0px;
	 border-top-right-radius: 0px;
	 border-bottom-left-radius: 0px;
	 border-bottom-right-radius: 0px;
	 padding-right: 3px;
	 min-width: 8ex;
	 width: 8ex !important;
	}
	div.cell.code_cell .input_prompt {
	 border-right: 2px solid rgba(191,11,55,.30);
	}
	div.cell.selected .prompt {
	 top: 0;
	}
	.edit_mode div.cell.selected .prompt {
	 top: 0;
	}
	.edit_mode div.cell.selected .prompt {
	 top: 0;
	}
	.run_this_cell {
	 visibility: hidden;
	 color: transparent;
	 padding-top: 0px;
	 padding-bottom: 0px;
	 padding-left: 3px;
	 padding-right: 12px;
	 width: 1.5ex;
	 width: 0ex;
	 background: transparent;
	 background-color: transparent;
	}
	div.code_cell:hover div.input .run_this_cell {
	 visibility: visible;
	}
	div.cell.code_cell.rendered.selected .run_this_cell:hover {
	 background-color: #e7e7e7;
	 background: #e7e7e7;
	 color: #c60030 !important;
	}
	div.cell.code_cell.rendered.unselected .run_this_cell:hover {
	 background-color: #e7e7e7;
	 background: #e7e7e7;
	 color: #c60030 !important;
	}
	i.fa-step-forward.fa {
	 display: inline-block;
	 font: normal normal normal 9px "FontAwesome";
	}
	.fa-step-forward:before {
	 content: "\f04b";
	}
	div.cell.selected.jupyter-soft-selected .run_this_cell,
	div.cell.selected.jupyter-soft-selected .run_this_cell:hover,
	div.cell.unselected.jupyter-soft-selected .run_this_cell:hover,
	div.cell.code_cell.rendered.selected.jupyter-soft-selected .run_this_cell:hover,
	div.cell.code_cell.rendered.unselected.jupyter-soft-selected .run_this_cell:hover {
	 background-color: #ffbbc3 !important;
	 background: #ffbbc3 !important;
	 color: #ffbbc3 !important;
	}
	div.output_wrapper {
	 background-color: #ffffff;
	 border: 0px;
	 left: 0px;
	 margin-bottom: 0em;
	 margin-top: 0em;
	 border-top-right-radius: 0px;
	 border-top-left-radius: 0px;
	}
	div.output_subarea.output_text.output_stream.output_stdout,
	div.output_subarea.output_text {
	 font-family: monospace, monospace;
	 font-size: 9pt !important;
	 line-height: 150% !important;
	 background-color: #ffffff;
	 color: #303030;
	 border-top-right-radius: 0px;
	 border-top-left-radius: 0px;
	}
	div.output_area pre {
	 font-family: monospace, monospace;
	 font-size: 9pt !important;
	 line-height: 151% !important;
	 color: #303030;
	 border-top-right-radius: 0px;
	 border-top-left-radius: 0px;
	}
	div.output_area {
	 display: -webkit-box;
	}
	div.output_html {
	 font-family: monospace, monospace;
	 font-size: 9pt;
	 color: #353535;
	 background-color: #ffffff;
	 background: #ffffff;
	}
	div.output_subarea {
	 overflow-x: auto;
	 padding: 0.8em !important;
	 -webkit-box-flex: 1;
	 -moz-box-flex: 1;
	 box-flex: 1;
	 flex: 1;
	}
	div.btn.btn-default.output_collapsed {
	 background: #d7d7d7;
	 background-color: #d7d7d7;
	 border-color: #d7d7d7;
	}
	div.btn.btn-default.output_collapsed:hover {
	 background: #d2d2d2;
	 background-color: #d2d2d2;
	 border-color: #d2d2d2;
	}
	div.prompt.output_prompt {
	 font-family: monospace, monospace;
	 font-size: 9pt !important;
	 font-weight: bold !important;
	 background-color: #ffffff;
	 color: transparent;
	 border-bottom-left-radius: 4px;
	 border-top-right-radius: 0px;
	 border-top-left-radius: 0px;
	 border-bottom-right-radius: 0px;
	 min-width: 11ex !important;
	 width: 11ex !important;
	 border-right: 2px solid #ffffff;
	}
	div.out_prompt_overlay.prompt {
	 font-family: monospace, monospace;
	 font-size: 9pt !important;
	 font-weight: bold !important;
	 background-color: #ffffff;
	 border-bottom-left-radius: 2px;
	 border-top-right-radius: 0px;
	 border-top-left-radius: 0px;
	 border-bottom-right-radius: 0px;
	 min-width: 11ex !important;
	 width: 11ex !important;
	 border-right: 2px solid #ffffff;
	 color: transparent;
	}
	div.out_prompt_overlay.prompt:hover {
	 background-color: #f5f5f5;
	 box-shadow: none !important;
	 border: none;
	 border-bottom-left-radius: 2px;
	 -webkit-border-: 2px;
	 -moz-border-radius: 2px;
	 border-top-right-radius: 0px;
	 border-top-left-radius: 0px;
	 min-width: 11ex !important;
	 width: 11ex !important;
	 border-right: 2px solid #f5f5f5 !important;
	}
	div.cell.code_cell .output_prompt {
	 border-right: 2px solid transparent;
	 color: transparent;
	}
	div.cell.selected .output_prompt,
	div.cell.selected .out_prompt_overlay.prompt {
	 border-left: 4px solid #e8e8e8;
	 border-radius: 0px !important;
	}
	.edit_mode div.cell.selected .output_prompt,
	.edit_mode div.cell.selected .out_prompt_overlay.prompt {
	 border-left: 4px solid #e8e8e8;
	 border-right: 2px solid #ffffff;
	 border-radius: 0px !important;
	}
	div.text_cell,
	div.text_cell_render pre,
	div.text_cell_render {
	 font-family: sans-serif;
	 font-size: 11.5pt;
	 line-height: 130% !important;
	 color: #353535;
	 background: #ffffff;
	 background-color: #ffffff;
	 border-radius: 0px;
	}
	div .text_cell_render {
	 padding: 0.4em 0.4em 0.4em 0.4em;
	}
	div.cell.text_cell .CodeMirror-lines {
	 padding-top: .7em !important;
	 padding-bottom: .4em !important;
	 padding-left: .5em !important;
	 padding-right: .5em !important;
	 margin-top: .4em;
	 margin-bottom: .3em;
	}
	div.cell.text_cell.unrendered div.input_area,
	div.cell.text_cell.rendered div.input_area {
	 background-color: #ffffff;
	 background: #ffffff;
	 border: 0px;
	 border-radius: 2px;
	}
	div.cell.text_cell .CodeMirror,
	div.cell.text_cell .CodeMirror pre {
	 line-height: 170% !important;
	}
	div.cell.text_cell.rendered.selected {
	 font-family: sans-serif;
	 line-height: 180% !important;
	 background: #ffffff;
	 background-color: #ffffff;
	 border-radius: 0px;
	}
	div.cell.text_cell.unrendered.selected {
	 font-family: sans-serif;
	 line-height: 180% !important;
	 background: #ffffff;
	 background-color: #ffffff;
	 border-radius: 0px;
	}
	div.cell.text_cell.selected {
	 font-family: sans-serif;
	 line-height: 180% !important;
	 background: #ffffff;
	 background-color: #ffffff;
	 border-radius: 0px;
	}
	.edit_mode div.cell.text_cell.selected {
	 font-family: sans-serif;
	 line-height: 180% !important;
	 background: #ffffff;
	 background-color: #ffffff;
	 border-radius: 0px;
	}
	.edit_mode div.cell.text_cell.selected:before {
	 height: 100%;
	 border-left: 4px solid #e8e8e8 !important;
	 border-radius: 5px !important;
	}
	div.cell.text_cell.selected .input {
	 border-left: 4px solid #e0e0e0 !important;
	 border-radius: 2px;
	}
	.edit_mode div.cell.text_cell.selected .input {
	 border-left: 4px solid tc-border-edit !important;
	 border-radius: 2px;
	}
	div.text_cell.unrendered,
	div.text_cell.unrendered.selected,
	div.edit_mode div.text_cell.unrendered {
	 font-family: sans-serif;
	 line-height: 180% !important;
	 background: #ffffff;
	 background-color: #ffffff;
	 border-radius: 0px;
	}
	div.cell.text_cell.selected:before {
	 background: #e0e0e0 !important;
	 border: 2px solid #e0e0e0;
	 border-top-left-radius: 2px;
	 border-bottom-left-radius: 2px;
	 position: absolute;
	 display: block;
	 top: 0px;
	 left: 0px;
	 width: 4px;
	 height: 100%;
	}
	.edit_mode div.cell.text_cell.selected:before {
	 background: #e8e8e8 !important;
	 border: 2px solid #e8e8e8 !important;
	 border-top-left-radius: 2px;
	 border-bottom-left-radius: 2px;
	 position: absolute;
	 display: block;
	 top: 0px;
	 left: 0px;
	 width: 4px;
	 height: 100%;
	}
	div.cell.text_cell .prompt {
	 background-color: #ffffff !important;
	 min-width: 8ex !important;
	 width: 8ex !important;
	 border-left: 4px solid #ffffff;
	 border-top-left-radius: 3px !important;
	 border-bottom-left-radius: 3px !important;
	 border-top-right-radius: 0px;
	 border-bottom-right-radius: 0px;
	}
	div.cell.text_cell.rendered .prompt {
	 font-family: monospace, monospace;
	 font-size: 9pt !important;
	 font-weight: normal;
	 color: transparent !important;
	 text-align: right !important;
	 min-width: 11ex !important;
	 width: 11ex !important;
	 background-color: #ffffff;
	 border-left: 4px solid #ffffff;
	 border-top-left-radius: 3px !important;
	 border-bottom-left-radius: 3px !important;
	 border-top-right-radius: 0px;
	 border-bottom-right-radius: 0px;
	}
	div.cell.text_cell.unrendered .prompt {
	 font-family: monospace, monospace;
	 font-size: 9pt !important;
	 font-weight: normal;
	 color: transparent !important;
	 text-align: right !important;
	 min-width: 11ex !important;
	 width: 11ex !important;
	 border-left: 4px solid #ffffff;
	 border-top-left-radius: 3px !important;
	 border-bottom-left-radius: 3px !important;
	 border-top-right-radius: 0px;
	 border-bottom-right-radius: 0px;
	}
	div.cell.text_cell.rendered.selected .prompt {
	 top: 0;
	 border-left: 4px solid #e8e8e8;
	 border-top-left-radius: 3px !important;
	 border-bottom-left-radius: 3px !important;
	 border-top-right-radius: 0px;
	 border-bottom-right-radius: 0px;
	}
	div.text_cell.unrendered.selected .prompt,
	div.text_cell.rendered.selected .prompt {
	 top: 0;
	 background: #ffffff;
	 border-left: 4px solid #e8e8e8;
	 border-top-left-radius: 3px !important;
	 border-bottom-left-radius: 3px !important;
	 border-top-right-radius: 0px;
	 border-bottom-right-radius: 0px;
	}
	div.rendered_html code {
	 font-family: monospace, monospace;
	 font-size: 11pt;
	 padding-top: 3px;
	 padding-left: 2px;
	 color: #303030;
	 background: #e5e5e5;
	 background-color: #e5e5e5;
	}
	pre,
	code,
	kbd,
	samp {
	 white-space: pre-wrap;
	}
	.well code,
	code {
	 font-family: monospace, monospace;
	 font-size: 11pt !important;
	 line-height: 180% !important;
	 color: #353535;
	 background: #e5e5e5;
	 background-color: #e5e5e5;
	 border-color: #e5e5e5;
	}
	kbd {
	 padding: 1px;
	 font-size: 11pt;
	 font-weight: 800;
	 color: #303030;
	 background-color: transparent !important;
	 border: 0;
	 box-shadow: none;
	}
	pre {
	 display: block;
	 padding: 8.5px;
	 margin: 0 0 9px;
	 font-size: 10.5pt;
	 line-height: 1.42857143;
	 color: #303030;
	 background-color: #e5e5e5;
	 border: 1px solid #e5e5e5;
	 border-radius: 2px;
	}
	div.rendered_html {
	 color: #353535;
	}
	.rendered_html * + ul {
	 margin-top: .4em;
	 margin-bottom: .3em;
	}
	.rendered_html * + p {
	 margin-top: .5em;
	 margin-bottom: .5em;
	}
	div.rendered_html pre {
	 font-family: monospace, monospace;
	 font-size: 11pt !important;
	 line-height: 180% !important;
	 color: #353535;
	 background: #e5e5e5;
	 background-color: #e5e5e5;
	 max-width: 80%;
	 border-radius: 0px;
	 border-left: 3px solid #e5e5e5;
	 max-width: 80%;
	 border-radius: 0px;
	 padding-left: 5px;
	 margin-left: 6px;
	}
	div.text_cell_render pre,
	div.text_cell_render code {
	 font-family: monospace, monospace;
	 font-size: 11pt !important;
	 line-height: 180% !important;
	 color: #353535;
	 background: #ffffff;
	 background-color: #ffffff;
	 max-width: 80%;
	 border-radius: 0px;
	 border-left: none;
	}
	div.text_cell_render pre {
	 border-left: 3px solid rgba(191,11,55,.30) !important;
	 max-width: 80%;
	 border-radius: 0px;
	 padding-left: 5px;
	 margin-left: 6px;
	}
	div.text_cell_render h1,
	div.rendered_html h1,
	div.text_cell_render h2,
	div.rendered_html h2,
	div.text_cell_render h3,
	div.rendered_html h3,
	div.text_cell_render h4,
	div.rendered_html h4,
	div.text_cell_render h5,
	div.rendered_html h5 {
	 font-family: sans-serif;
	 margin: 0.4em .2em .3em .2em !important;
	}
	.rendered_html h1:first-child,
	.rendered_html h2:first-child,
	.rendered_html h3:first-child,
	.rendered_html h4:first-child,
	.rendered_html h5:first-child,
	.rendered_html h6:first-child {
	 margin-top: 0.2em !important;
	 margin-bottom: 0.2em !important;
	}
	.rendered_html h1,
	.text_cell_render h1 {
	 color: #b9002d;
	 font-size: 200%;
	 text-align: left;
	 font-style: normal;
	 font-weight: normal;
	}
	.rendered_html h2,
	.text_cell_render h2 {
	 text-align: left;
	 font-size: 170%;
	 color: #b9002d;
	 font-style: normal;
	 font-weight: normal;
	}
	.rendered_html h3,
	.text_cell_render h3 {
	 font-size: 140%;
	 color: #b9002d;
	 font-style: normal;
	 font-weight: normal;
	}
	.rendered_html h4,
	.text_cell_render h4 {
	 font-size: 110%;
	 color: #b9002d;
	 font-style: normal;
	 font-weight: normal;
	}
	.rendered_html h5,
	.text_cell_render h5 {
	 font-size: 100%;
	 color: #b9002d;
	 font-style: normal;
	 font-weight: normal;
	}
	hr {
	 margin-top: 8px;
	 margin-bottom: 10px;
	 border: 0;
	 border-top: 1px solid #b9002d;
	}
	.rendered_html hr {
	 color: #b9002d;
	 background-color: #b9002d;
	 margin-right: 2em;
	}
	#complete > select > option:hover {
	 background: #d4d4d4;
	 background-color: #d4d4d4;
	}
	div#_vivaldi-spatnav-focus-indicator._vivaldi-spatnav-focus-indicator {
	 position: absolute;
	 z-index: 9999999999;
	 top: 0px;
	 left: 0px;
	 box-shadow: none;
	 pointer-events: none;
	 border-radius: 2px;
	}
	.rendered_html tr,
	.rendered_html th,
	.rendered_html td {
	 text-align: left;
	 vertical-align: middle;
	 padding: 0.42em 0.47em;
	 line-height: normal;
	 white-space: normal;
	 max-width: none;
	 border: none;
	}
	.rendered_html td {
	 font-family: sans-serif !important;
	 font-size: 9.5pt;
	}
	.rendered_html table {
	 font-family: sans-serif !important;
	 margin-left: 8px;
	 margin-right: auto;
	 border: none;
	 border-collapse: collapse;
	 border-spacing: 0;
	 color: #353535;
	 table-layout: fixed;
	}
	.rendered_html thead {
	 font-family: sans-serif !important;
	 font-size: 10.5pt !important;
	 background: #d31242;
	 color: #ffffff;
	 border-bottom: 1px solid #d31242;
	 vertical-align: bottom;
	}
	.rendered_html tbody tr:nth-child(odd) {
	 background: #efefef;
	}
	.rendered_html tbody tr {
	 background: #e7e7e7;
	}
	.rendered_html tbody tr:hover:nth-child(odd) {
	 background: #ececec;
	}
	.rendered_html tbody tr:hover {
	 background: #e5e5e5;
	}
	.rendered_html * + table {
	 margin-top: .05em;
	}
	div.widget-area {
	 background-color: #ffffff;
	 background: #ffffff;
	 color: #303030;
	}
	div.widget-area a {
	 font-family: sans-serif;
	 font-size: 10.5pt;
	 font-weight: normal;
	 font-style: normal;
	 color: #070707;
	 text-shadow: none !important;
	}
	div.widget-area a:hover,
	div.widget-area a:focus {
	 font-family: sans-serif;
	 font-size: 10.5pt;
	 font-weight: normal;
	 font-style: normal;
	 color: #000000;
	 background: rgba(61,59,73,.10);
	 background-color: rgba(61,59,73,.10);
	 border-color: transparent;
	 background-image: none;
	 text-shadow: none !important;
	}
	div.widget_item.btn-group > button.btn.btn-default.widget-combo-btn,
	div.widget_item.btn-group > button.btn.btn-default.widget-combo-btn:hover {
	 background: #e1e1e1;
	 background-color: #e1e1e1;
	 border: 2px solid #e1e1e1 !important;
	 font-size: inherit;
	 z-index: 0;
	}
	div.jupyter-widgets.widget-hprogress.widget-hbox {
	 display: inline-table !important;
	 width: 38% !important;
	 margin-left: 10px;
	}
	div.jupyter-widgets.widget-hprogress.widget-hbox .widget-label,
	div.widget-hbox .widget-label,
	.widget-hbox .widget-label,
	.widget-inline-hbox .widget-label,
	div.widget-label {
	 text-align: -webkit-auto !important;
	 margin-left: 15px !important;
	 max-width: 240px !important;
	 min-width: 100px !important;
	 vertical-align: text-top !important;
	 color: #303030 !important;
	 font-size: 14px !important;
	}
	.widget-hprogress .progress {
	 flex-grow: 1;
	 height: 20px;
	 margin-top: auto;
	 margin-left: 12px;
	 margin-bottom: auto;
	 width: 300px;
	}
	.progress {
	 overflow: hidden;
	 height: 22px;
	 margin-bottom: 10px;
	 padding-left: 10px;
	 background-color: #d6d6d6 !important;
	 border-radius: 2px;
	 -webkit-box-shadow: none;
	 box-shadow: none;
	 z-index: 10;
	}
	.progress-bar-danger {
	 background-color: #b9002d !important;
	}
	.progress-bar-info {
	 background-color: #3e5e9c !important;
	}
	.progress-bar-warning {
	 background-color: #d05a0b !important;
	}
	.progress-bar-success {
	 background-color: #018c8c !important;
	}
	.widget-select select {
	 margin-left: 12px;
	}
	.rendered_html :link {
	 font-family: sans-serif;
	 font-size: 100%;
	 color: #d31242;
	 text-decoration: underline;
	}
	.rendered_html :visited,
	.rendered_html :visited:active,
	.rendered_html :visited:focus {
	 color: #df1a4b;
	}
	.rendered_html :visited:hover,
	.rendered_html :link:hover {
	 font-family: sans-serif;
	 font-size: 100%;
	 color: #bb012f;
	}
	div.cell.text_cell a.anchor-link:link {
	 font-size: inherit;
	 text-decoration: none;
	 padding: 0px 20px;
	 visibility: none;
	 color: rgba(191,11,55,.8);
	}
	div.cell.text_cell a.anchor-link:link:hover {
	 font-size: inherit;
	 color: #007dc5;
	}
	.navbar-text {
	 margin-top: 4px;
	 margin-bottom: 0px;
	}
	#clusters > a {
	 color: #007dc5;
	 text-decoration: underline;
	 cursor: auto;
	}
	#clusters > a:hover {
	 color: #007dc5;
	 text-decoration: underline;
	 cursor: auto;
	}
	#nbextensions-configurator-container > div.row.container-fluid.nbext-selector > h3 {
	 font-size: 17px;
	 margin-top: 5px;
	 margin-bottom: 8px;
	 height: 24px;
	 padding: 4px 0 4px 0;
	}
	div#nbextensions-configurator-container.container,
	#nbextensions-configurator-container.container {
	 width: 100%;
	 margin-right: auto;
	 margin-left: auto;
	}
	div.nbext-selector > nav > .nav > li > a {
	 font-family: sans-serif;
	 font-size: 10.5pt;
	 padding: 2px 5px;
	}
	div.nbext-selector > nav > .nav > li > a:hover {
	 background: transparent;
	}
	div.nbext-selector > nav > .nav > li:hover {
	 background-color: rgba(61,59,73,.10) !important;
	 background: rgba(61,59,73,.10) !important;
	}
	div.nbext-selector > nav > .nav > li.active:hover {
	 background: transparent !important;
	 background-color: transparent !important;
	}
	.nav-pills > li.active > a,
	.nav-pills > li.active > a:active,
	.nav-pills > li.active > a:hover,
	.nav-pills > li.active > a:focus {
	 color: #000000;
	 background-color: rgba(61,59,73,.10) !important;
	 background: rgba(61,59,73,.10) !important;
	 -webkit-backface-visibility: hidden;
	 -webkit-font-smoothing: subpixel-antialiased !important;
	}
	div.nbext-readme > .nbext-readme-contents > .rendered_html {
	 font-family: sans-serif;
	 font-size: 11.5pt;
	 line-height: 145%;
	 padding: 1em 1em;
	 color: #353535;
	 background-color: #ffffff;
	 -webkit-box-shadow: none;
	 -moz-box-shadow: none;
	 box-shadow: none;
	}
	.nbext-icon,
	.nbext-desc,
	.nbext-compat-div,
	.nbext-enable-btns,
	.nbext-params {
	 margin-bottom: 8px;
	 font-size: 11.5pt;
	}
	div.nbext-readme > .nbext-readme-contents {
	 padding: 0;
	 overflow-y: hidden;
	}
	div.nbext-readme > .nbext-readme-contents:not(:empty) {
	 margin-top: 0.5em;
	 margin-bottom: 2em;
	 border: none;
	 border-top-color: #828892;
	}
	.nbext-showhide-incompat {
	 padding-bottom: 0.5em;
	 color: #4a4a4a;
	 font-size: 10.5pt;
	}
	.nbext-filter-menu.dropdown-menu > li > a:hover,
	.nbext-filter-menu.dropdown-menu > li > a:focus,
	.nbext-filter-menu.dropdown-menu > li > a.ui-state-focus {
	 color: #000000 !important;
	 background-color: #d4d4d4 !important;
	 background: #d4d4d4 !important;
	 border-color: #d4d4d4 !important;
	}
	.nbext-filter-input-wrap > .nbext-filter-input-subwrap,
	.nbext-filter-input-wrap > .nbext-filter-input-subwrap > input {
	 border: none;
	 outline: none;
	 background-color: transparent;
	 padding: 0;
	 vertical-align: middle;
	 margin-top: -2px;
	}
	span.rendered_html code {
	 background-color: transparent;
	 color: #070707;
	}
	#nbextensions-configurator-container > div.row.container-fluid.nbext-selector {
	 padding-left: 0px;
	 padding-right: 0px;
	}
	.nbext-filter-menu {
	 max-height: 50vh;
	 overflow-y: auto;
	 outline: none;
	 border: 2px solid #dedede;
	}
	.nbext-filter-menu:hover {
	 border: 2px solid #dedede;
	}
	.alert-warning {
	 background-color: #f4f4f4;
	 border-color: #f4f4f4;
	 color: #070707;
	}
	.notification_widget.danger {
	 color: #ffffff;
	 background-color: #b9002d;
	 border-color: #b9002d;
	 padding-right: 5px;
	}
	#nbextensions-configurator-container > div.nbext-buttons.tree-buttons.no-padding.pull-right > span > button {
	 border: none !important;
	}
	button#refresh_running_list {
	 border: none !important;
	}
	mark,
	.mark {
	 background-color: #ffffff;
	 color: #353535;
	 padding: .15em;
	}
	a.text-warning,
	a.text-warning:hover {
	 color: #828892;
	}
	a.text-warning.bg-warning {
	 background-color: #ffffff;
	}
	span.bg-success.text-success {
	 background-color: transparent;
	 color: #488f0c;
	}
	span.bg-danger.text-danger {
	 background-color: #ffffff;
	 color: #b9002d;
	}
	.has-success .input-group-addon {
	 color: #488f0c;
	 border-color: transparent;
	 background: inherit;
	 background-color: rgba(83,180,115,.10);
	}
	.has-success .form-control {
	 border-color: #488f0c;
	 -webkit-box-shadow: inset 0 1px 1px rgba(0,0,0,0.025);
	 box-shadow: inset 0 1px 1px rgba(0,0,0,0.025);
	}
	.has-error .input-group-addon {
	 color: #b9002d;
	 border-color: transparent;
	 background: inherit;
	 background-color: rgba(192,57,67,.10);
	}
	.has-error .form-control {
	 border-color: #b9002d;
	 -webkit-box-shadow: inset 0 1px 1px rgba(0,0,0,0.025);
	 box-shadow: inset 0 1px 1px rgba(0,0,0,0.025);
	}
	.kse-input-group-pretty > kbd {
	 font-family: monospace, monospace;
	 color: #070707;
	 font-weight: normal;
	 background: transparent;
	}
	.kse-input-group-pretty > kbd {
	 font-family: monospace, monospace;
	 color: #070707;
	 font-weight: normal;
	 background: transparent;
	}
	div.nbext-enable-btns .btn[disabled],
	div.nbext-enable-btns .btn[disabled]:hover,
	.btn-default.disabled,
	.btn-default[disabled] {
	 background: #e6e6e6;
	 background-color: #e6e6e6;
	 color: #000000;
	}
	label#Keyword-Filter {
	 display: none;
	}
	.input-group .nbext-list-btn-add,
	.input-group-btn:last-child > .btn-group > .btn {
	 background: #e4e4e4;
	 background-color: #e4e4e4;
	 border-color: #e4e4e4;
	 border: 2px solid #e4e4e4;
	}
	.input-group .nbext-list-btn-add:hover,
	.input-group-btn:last-child > .btn-group > .btn:hover {
	 background: #dfdfdf;
	 background-color: #dfdfdf;
	 border-color: #dfdfdf;
	 border: 2px solid #dfdfdf;
	}
	#notebook-container > div.cell.code_cell.rendered.selected > div.widget-area > div.widget-subarea > div > div.widget_item.btn-group > button.btn.btn-default.dropdown-toggle.widget-combo-carrot-btn {
	 background: #e4e4e4;
	 background-color: #e4e4e4;
	 border-color: #e4e4e4;
	}
	#notebook-container > div.cell.code_cell.rendered.selected > div.widget-area > div.widget-subarea > div > div.widget_item.btn-group > button.btn.btn-default.dropdown-toggle.widget-combo-carrot-btn:hover {
	 background: #dfdfdf;
	 background-color: #dfdfdf;
	 border-color: #dfdfdf;
	}
	.ui-widget-content {
	 background: #e9e9e9;
	 background-color: #e9e9e9;
	 border: 2px solid #e9e9e9;
	 color: #070707;
	}
	div.collapsible_headings_toggle {
	 color: rgba(61,59,73,.15) !important;
	}
	div.collapsible_headings_toggle:hover {
	 color: #d31242 !important;
	}
	.collapsible_headings_toggle .h1,
	.collapsible_headings_toggle .h2,
	.collapsible_headings_toggle .h3,
	.collapsible_headings_toggle .h4,
	.collapsible_headings_toggle .h5,
	.collapsible_headings_toggle .h6 {
	 margin: 0.3em .4em 0em 0em !important;
	 line-height: 1.2 !important;
	}
	div.collapsible_headings_toggle .fa-caret-down:before,
	div.collapsible_headings_toggle .fa-caret-right:before {
	 font-size: xx-large;
	 transition: transform 1000ms;
	 transform: none !important;
	}
	.collapsible_headings_collapsed.collapsible_headings_ellipsis .rendered_html h1:after,
	.collapsible_headings_collapsed.collapsible_headings_ellipsis .rendered_html h2:after,
	.collapsible_headings_collapsed.collapsible_headings_ellipsis .rendered_html h3:after,
	.collapsible_headings_collapsed.collapsible_headings_ellipsis .rendered_html h4:after,
	.collapsible_headings_collapsed.collapsible_headings_ellipsis .rendered_html h5:after,
	.collapsible_headings_collapsed.collapsible_headings_ellipsis .rendered_html h6:after {
	 position: absolute;
	 right: 0;
	 bottom: 20% !important;
	 content: "[\002026]";
	 color: rgba(61,59,73,.15) !important;
	 padding: 0.5em 0em 0em 0em !important;
	}
	.collapsible_headings_ellipsis .rendered_html h1,
	.collapsible_headings_ellipsis .rendered_html h2,
	.collapsible_headings_ellipsis .rendered_html h3,
	.collapsible_headings_ellipsis .rendered_html h4,
	.collapsible_headings_ellipsis .rendered_html h5,
	.collapsible_headings_ellipsis .rendered_html h6,
	.collapsible_headings_toggle .fa {
	 transition: transform 1000ms !important;
	 -webkit-transform: inherit !important;
	 -moz-transform: inherit !important;
	 -ms-transform: inherit !important;
	 -o-transform: inherit !important;
	 transform: inherit !important;
	 padding-right: 0px !important;
	}
	input.raw_input {
	 font-family: monospace, monospace;
	 font-size: 11pt !important;
	 color: #303030;
	 background-color: #e5e5e5;
	 border-color: #e2e2e2;
	 background: #e2e2e2;
	 width: auto;
	 vertical-align: baseline;
	 padding: 0em 0.25em;
	 margin: 0em 0.25em;
	 -webkit-box-shadow: none;
	 box-shadow: none;
	}
	#toc-wrapper {
	 z-index: 90;
	 position: fixed !important;
	 display: flex;
	 flex-direction: column;
	 overflow: hidden;
	 padding: 10px;
	 border-style: solid;
	 border-width: thin;
	 border-right-width: medium !important;
	 background-color: #ffffff !important;
	}
	#toc-header,
	.modal-header {
	 padding-top: 36px !important;
	}
	#toc-wrapper.ui-draggable.ui-resizable.sidebar-wrapper {
	 border-color: rgba(61,59,73,.10) !important;
	}
	#toc a,
	#navigate_menu a,
	.toc {
	 color: #070707 !important;
	}
	#toc li > span:hover {
	 background-color: #d4d4d4 !important;
	}
	#toc a:hover,
	#navigate_menu a:hover,
	.toc {
	 color: #000000 !important;
	}
	#toc-wrapper .toc-item-num {
	 color: #d31242 !important;
	}
	audio,
	video {
	 display: inline;
	 vertical-align: middle;
	 align-content: center;
	 margin-left: 20%;
	}
	.cmd-palette .modal-body {
	 padding: 0px;
	 margin: 0px;
	}
	.cmd-palette form {
	 background: #e5e5e5;
	 background-color: #e5e5e5;
	}
	.typeahead-field input:last-child,
	.typeahead-hint {
	 background: #e5e5e5;
	 background-color: #e5e5e5;
	 z-index: 1;
	}
	.typeahead-field input {
	 font-family: sans-serif;
	 color: #303030;
	 border: none;
	 font-size: 28pt;
	 display: inline-block;
	 line-height: inherit;
	 padding: 3px 10px;
	 height: 70px;
	}
	.typeahead-select {
	 background-color: #e5e5e5;
	}
	body > div.modal.cmd-palette.typeahead-field {
	 display: table;
	 border-collapse: separate;
	 background-color: #f4f4f4;
	}
	.typeahead-container button {
	 font-family: sans-serif;
	 font-size: 28pt;
	 background-color: #e4e4e4;
	 border: none;
	 display: inline-block;
	 line-height: inherit;
	 padding: 3px 10px;
	 height: 70px;
	}
	.typeahead-search-icon {
	 min-width: 40px;
	 min-height: 55px;
	 display: block;
	 vertical-align: middle;
	 text-align: center;
	}
	.typeahead-container button:focus,
	.typeahead-container button:hover {
	 color: #000000;
	 background-color: #dfdfdf;
	 border-color: #e4e4e4;
	}
	.typeahead-list > li.typeahead-group.active > a,
	.typeahead-list > li.typeahead-group > a,
	.typeahead-list > li.typeahead-group > a:focus,
	.typeahead-list > li.typeahead-group > a:hover {
	 display: none;
	}
	.typeahead-dropdown > li > a,
	.typeahead-list > li > a {
	 color: #070707;
	 text-decoration: none;
	}
	.typeahead-dropdown,
	.typeahead-list {
	 font-family: sans-serif;
	 font-size: 11.5pt;
	 color: #070707;
	 background-color: #ffffff;
	 border: none;
	 background-clip: padding-box;
	 margin-top: 0px;
	 padding: 3px 2px 3px 0px;
	 line-height: 1.7;
	}
	.typeahead-dropdown > li.active > a,
	.typeahead-dropdown > li > a:focus,
	.typeahead-dropdown > li > a:hover,
	.typeahead-list > li.active > a,
	.typeahead-list > li > a:focus,
	.typeahead-list > li > a:hover {
	 color: #000000;
	 background-color: #f4f4f4;
	 border-color: #f4f4f4;
	}
	.command-shortcut:before {
	 content: "(command)";
	 padding-right: 3px;
	 color: #828892;
	}
	.edit-shortcut:before {
	 content: "(edit)";
	 padding-right: 3px;
	 color: #828892;
	}
	ul.typeahead-list i {
	 margin-left: 1px;
	 width: 18px;
	 margin-right: 10px;
	}
	ul.typeahead-list {
	 max-height: 50vh;
	 overflow: auto;
	}
	.typeahead-list > li {
	 position: relative;
	 border: none;
	}
	div.input.typeahead-hint,
	input.typeahead-hint,
	body > div.modal.cmd-palette.in > div > div > div > form > div > div.typeahead-field > span.typeahead-query > input.typeahead-hint {
	 color: #828892 !important;
	 background-color: transparent;
	 padding: 3px 10px;
	}
	.typeahead-dropdown > li > a,
	.typeahead-list > li > a {
	 display: block;
	 padding: 5px;
	 clear: both;
	 font-weight: 400;
	 line-height: 1.7;
	 border: 1px solid #ffffff;
	 border-bottom-color: rgba(61,59,73,.15);
	}
	body > div.modal.cmd-palette.in > div {
	 min-width: 750px;
	 margin: 150px auto;
	}
	.typeahead-container strong {
	 font-weight: bolder;
	 color: #d31242;
	}
	#find-and-replace #replace-preview .match,
	#find-and-replace #replace-preview .insert {
	 color: #ffffff;
	 background-color: #c60030;
	 border-color: #c60030;
	 border-style: solid;
	 border-width: 1px;
	 border-radius: 0px;
	}
	#find-and-replace #replace-preview .replace .match {
	 background-color: #b9002d;
	 border-color: #b9002d;
	 border-radius: 0px;
	}
	#find-and-replace #replace-preview .replace .insert {
	 background-color: #488f0c;
	 border-color: #488f0c;
	 border-radius: 0px;
	}
	.shortcut_key,
	span.shortcut_key {
	 display: inline-block;
	 width: 16ex;
	 text-align: right;
	 font-family: monospace;
	}
	.jupyter-keybindings {
	 padding: 1px;
	 line-height: 24px;
	 border-bottom: 1px solid rgba(61,59,73,.10);
	}
	.jupyter-keybindings i {
	 background: #e5e5e5;
	 font-size: small;
	 padding: 5px;
	 margin-left: 7px;
	}
	div#short-key-bindings-intro.well,
	.well {
	 background-color: #e4e4e4;
	 border: 1px solid #e4e4e4;
	 color: #070707;
	 border-radius: 2px;
	 -webkit-box-shadow: none;
	 box-shadow: none;
	}
	#texteditor-backdrop {
	 background: #ffffff;
	 background-color: #ffffff;
	}
	#texteditor-backdrop #texteditor-container .CodeMirror-gutter,
	#texteditor-backdrop #texteditor-container .CodeMirror-gutters {
	 background: #e0e1e3;
	 background-color: #e0e1e3;
	 color: #6a737d;
	}
	.edit_app #menubar .navbar {
	 margin-bottom: 0px;
	}
	#texteditor-backdrop #texteditor-container {
	 padding: 0px;
	 background-color: #f1f1f1;
	 box-shadow: none;
	}
	.terminal-app {
	 background: #ffffff;
	}
	.terminal-app .terminal {
	 font-family: monospace, monospace;
	 font-size: 11pt;
	 line-height: 180%;
	 color: #303030;
	 background: #f1f1f1;
	 padding: 0.4em;
	 border-radius: 2px;
	 -webkit-box-shadow: none;
	 box-shadow: none;
	}
	.terminal .xterm-viewport {
	 background-color: #f1f1f1;
	 color: #303030;
	 overflow-y: auto;
	}
	.terminal .xterm-color-0 {
	 color: #d31242;
	}
	.terminal .xterm-color-1 {
	 color: #d3305e;
	}
	.terminal .xterm-color-2 {
	 color: #7d4bb3;
	}
	.terminal .xterm-color-3 {
	 color: #d3305e;
	}
	.terminal .xterm-color-4 {
	 color: #f65d22;
	}
	.terminal .xterm-color-5 {
	 color: #488f0c;
	}
	.terminal .xterm-color-6 {
	 color: #d3305e;
	}
	.terminal .xterm-color-7 {
	 color: #3275d5;
	}
	.terminal .xterm-color-8 {
	 color: #007dc5;
	}
	.terminal .xterm-color-9 {
	 color: #488f0c;
	}
	.terminal .xterm-color-10 {
	 color: #d3305e;
	}
	.terminal .xterm-color-14 {
	 color: #d3305e;
	}
	.terminal .xterm-bg-color-15 {
	 background-color: #f1f1f1;
	}
	.terminal:not(.xterm-cursor-style-underline):not(.xterm-cursor-style-bar) .terminal-cursor {
	 background-color: #d31242;
	 color: #f1f1f1;
	}
	.terminal:not(.focus) .terminal-cursor {
	 outline: 1px solid #d31242;
	 outline-offset: -1px;
	}
	.celltoolbar {
	 font-size: 100%;
	 padding-top: 3px;
	 border-color: transparent;
	 border-bottom: thin solid #828892;
	 background: transparent;
	}
	.cell-tag,
	.tags-input input,
	.tags-input button {
	 color: #070707;
	 background-color: #ffffff;
	 background-image: none;
	 border: 1px solid #070707;
	 border-radius: 1px;
	 box-shadow: none;
	 width: inherit;
	 font-size: inherit;
	 height: 22px;
	 line-height: 22px;
	}
	#notebook-container > div.cell.code_cell.rendered.selected > div.input > div.inner_cell > div.ctb_hideshow.ctb_show > div > div > button,
	#notebook-container > div.input > div.inner_cell > div.ctb_hideshow.ctb_show > div > div > button {
	 font-size: 10pt;
	 color: #070707;
	 background-color: #ffffff;
	 background-image: none;
	 border: 1px solid #070707;
	 border-radius: 1px;
	 box-shadow: none;
	 width: inherit;
	 font-size: inherit;
	 height: 22px;
	 line-height: 22px;
	}
	div#pager #pager-contents {
	 background: #ffffff !important;
	 background-color: #ffffff !important;
	}
	div#pager pre {
	 color: #303030 !important;
	 background: #f1f1f1 !important;
	 background-color: #f1f1f1 !important;
	 padding: 0.4em;
	}
	div#pager .ui-resizable-handle {
	 top: 0px;
	 height: 8px;
	 background: #d31242 !important;
	 border-top: 1px solid #d31242;
	 border-bottom: 1px solid #d31242;
	}
	div.CodeMirror,
	div.CodeMirror pre {
	 font-family: monospace, monospace;
	 font-size: 11pt;
	 line-height: 180%;
	 color: #303030;
	}
	div.CodeMirror-lines {
	 padding-bottom: .9em;
	 padding-left: .5em;
	 padding-right: 1.5em;
	 padding-top: .7em;
	}
	span.ansiblack,
	.ansi-black-fg {
	 color: #9f0027;
	}
	span.ansiblue,
	.ansi-blue-fg,
	.ansi-blue-intense-fg {
	 color: #488f0c;
	}
	span.ansigray,
	.ansi-gray-fg,
	.ansi-gray-intense-fg {
	 color: #f65d22;
	}
	span.ansigreen,
	.ansi-green-fg {
	 color: #3f3d46;
	}
	.ansi-green-intense-fg {
	 color: #f65d22;
	}
	span.ansipurple,
	.ansi-purple-fg,
	.ansi-purple-intense-fg {
	 color: #7d4bb3;
	}
	span.ansicyan,
	.ansi-cyan-fg,
	.ansi-cyan-intense-fg {
	 color: #7d4bb3;
	}
	span.ansiyellow,
	.ansi-yellow-fg,
	.ansi-yellow-intense-fg {
	 color: #f65d22;
	}
	span.ansired,
	.ansi-red-fg,
	.ansi-red-intense-fg {
	 color: #d3305e;
	}
	div.output-stderr {
	 background-color: #ebb5b7;
	}
	div.output-stderr pre {
	 color: #000000;
	}
	div.js-error {
	 color: #d3305e;
	}
	.ipython_tooltip {
	 font-family: monospace, monospace;
	 font-size: 11pt;
	 line-height: 180%;
	 border: 2px solid #e1e1e1;
	 background: #f5f5f5;
	 background-color: #f5f5f5;
	 border-radius: 2px;
	 overflow-x: visible;
	 overflow-y: visible;
	 box-shadow: none;
	 position: absolute;
	 z-index: 1000;
	}
	.ipython_tooltip .tooltiptext pre {
	 font-family: monospace, monospace;
	 font-size: 11pt;
	 line-height: 180%;
	 background: #f5f5f5;
	 background-color: #f5f5f5;
	 color: #303030;
	 overflow-x: visible;
	 overflow-y: visible;
	 max-width: 900px;
	}
	div#tooltip.ipython_tooltip {
	 overflow-x: wrap;
	 overflow-y: visible;
	 max-width: 800px;
	}
	div.tooltiptext.bigtooltip {
	 overflow-x: visible;
	 overflow-y: scroll;
	 height: 400px;
	 max-width: 800px;
	}
	.cm-s-ipython.CodeMirror {
	 font-family: monospace, monospace;
	 font-size: 11pt;
	 background: #f1f1f1;
	 color: #303030;
	 border-radius: 2px;
	 font-style: normal;
	 font-weight: normal;
	}
	.cm-s-ipython div.CodeMirror-selected {
	 background: #e0e1e3;
	}
	.cm-s-ipython .CodeMirror-gutters {
	 background: #e0e1e3;
	 border: none;
	 border-radius: 0px;
	 width: 36px;
	}
	.cm-s-ipython .CodeMirror-linenumber {
	 color: #6a737d;
	}
	.CodeMirror-sizer {
	 margin-left: 40px;
	}
	.CodeMirror-linenumber,
	div.CodeMirror-linenumber,
	.CodeMirror-gutter.CodeMirror-linenumberdiv.CodeMirror-gutter.CodeMirror-linenumber {
	 padding-right: 1px;
	 margin-left: 0px;
	 margin: 0px;
	 width: 26px !important;
	 padding: 0px;
	 text-align: right;
	}
	.CodeMirror-linenumber {
	 color: #6a737d;
	}
	.cm-s-ipython .CodeMirror-cursor {
	 border-left: 2px solid #1e70c7 !important;
	}
	.cm-s-ipython span.cm-comment {
	 color: #909090;
	 font-style: italic;
	}
	.cm-s-ipython span.cm-atom {
	 color: #007dc5;
	}
	.cm-s-ipython span.cm-number {
	 color: #f65d22;
	}
	.cm-s-ipython span.cm-property {
	 color: #3f3d46;
	}
	.cm-s-ipython span.cm-attribute {
	 color: #3f3d46;
	}
	.cm-s-ipython span.cm-keyword {
	 color: #7d4bb3;
	 font-weight: normal;
	}
	.cm-s-ipython span.cm-string {
	 color: #488f0c;
	}
	.cm-s-ipython span.cm-meta {
	 color: #007dc5;
	 font-weight: bold;
	 font-style: italic;
	}
	.cm-s-ipython span.cm-operator {
	 color: #007dc5;
	}
	.cm-s-ipython span.cm-builtin {
	 color: #d3305e;
	}
	.cm-s-ipython span.cm-variable {
	 color: #3f3d46;
	}
	.cm-s-ipython span.cm-variable-2 {
	 color: #d3305e;
	}
	.cm-s-ipython span.cm-variable-3 {
	 color: #7d4bb3;
	}
	.cm-s-ipython span.cm-def {
	 color: #3275d5;
	 font-weight: bold;
	 font-style: italic;
	}
	.cm-s-ipython span.cm-error {
	 background: rgba(191,11,55,.10);
	}
	.cm-s-ipython span.cm-tag {
	 color: #9f0027;
	}
	.cm-s-ipython span.cm-link {
	 color: #007dc5;
	}
	.cm-s-ipython span.cm-storage {
	 color: #007dc5;
	}
	.cm-s-ipython span.cm-entity {
	 color: #d3305e;
	}
	.cm-s-ipython span.cm-quote {
	 color: #488f0c;
	}
	div.CodeMirror span.CodeMirror-matchingbracket {
	 color: #000000;
	 background-color: rgba(90,160,18,.40);
	}
	div.CodeMirror span.CodeMirror-nonmatchingbracket {
	 color: #000000;
	 background: rgba(191,11,55,.10) !important;
	}
	.cm-header-1 {
	 font-size: 215%;
	}
	.cm-header-2 {
	 color: #3f3d46 !important;
	 font-size: 180%;
	}
	.cm-header-3 {
	 color: #3f3d46 !important;
	 font-size: 150%;
	}
	.cm-header-4 {
	 color: #3f3d46 !important;
	 font-size: 120%;
	}
	.cm-header-5 {
	 color: #3f3d46 !important;
	 font-size: 100%;
	}
	.cm-s-default .cm-hr {
	 color: #007dc5;
	}
	div.cell.text_cell .cm-s-default .cm-header {
	 font-family: sans-serif;
	 font-weight: normal;
	 color: #b9002d !important;
	 margin-top: 0.3em !important;
	 margin-bottom: 0.3em !important;
	}
	div.cell.text_cell .cm-s-default span.cm-variable-2 {
	 color: #353535 !important;
	}
	div.cell.text_cell .cm-s-default span.cm-variable-3 {
	 color: #7d4bb3 !important;
	}
	.cm-s-default span.cm-comment {
	 color: #909090;
	}
	.cm-s-default .cm-tag {
	 color: #007dc5;
	}
	.cm-s-default .cm-builtin {
	 color: #d3305e;
	}
	.cm-s-default .cm-string {
	 color: #488f0c;
	}
	.cm-s-default .cm-keyword {
	 color: #7d4bb3;
	}
	.cm-s-default .cm-number {
	 color: #f65d22;
	}
	.cm-s-default .cm-error {
	 color: #007dc5;
	}
	.cm-s-default .cm-link {
	 color: #007dc5;
	}
	.cm-s-default .cm-atom {
	 color: #f65d22;
	}
	.CodeMirror-cursor {
	 border-left: 2px solid #1e70c7 !important;
	 border-right: none;
	 width: 0;
	}
	.cm-s-default div.CodeMirror-selected {
	 background: #e0e1e3;
	}
	.cm-s-default .cm-selected {
	 background: #e0e1e3;
	}
	.MathJax_Display,
	.MathJax {
	 border: 0 !important;
	 font-size: 100% !important;
	 text-align: center !important;
	 margin: 0em !important;
	 line-height: 2.25 !important;
	}
	.MathJax:focus,
	body :focus .MathJax {
	 display: inline-block !important;
	}
	.MathJax:focus,
	body :focus .MathJax {
	 display: inline-block !important;
	}
	.completions {
	 position: absolute;
	 z-index: 110;
	 overflow: hidden;
	 border: medium solid rgba(191,11,55,.30);
	 box-shadow: none;
	 line-height: 1;
	}
	.completions select {
	 background: #f1f1f1;
	 background-color: #f1f1f1;
	 outline: none;
	 border: none;
	 padding: 0px;
	 margin: 0px;
	 margin-left: 2px;
	 overflow: auto;
	 font-family: monospace, monospace;
	 font-size: 11pt;
	 color: #303030;
	 width: auto;
	}
	div#maintoolbar {
	 display: none !important;
	}
	#header-container {
	 display: none !important;
	}
	
</style>
<script>
    MathJax.Hub.Config({
        "HTML-CSS": {
            /*preferredFont: "TeX",*/
            /*availableFonts: ["TeX", "STIX"],*/
            styles: {
                scale: 100,
                ".MathJax_Display": {
                    "font-size": "100%",
                }
            }
        }
    });
</script>
    